from graphics_and_games_klassen import Ereignisbehandlung, Dreieck, Rechteck, Text

class Wasserkocheranzeige(Ereignisbehandlung):
    """
    Klasse zur Beschreibung der Wasserkocheranzeige.
    """

    def __init__(self):
        """
        Initialisiert die Attribute und die benötigten Formen
        """
        super().__init__()

        self.istEingeschaltet = False
        
        d = Dreieck()
        d.FarbeSetzen("grau")
        d.PositionSetzen(450, 175)
        d.GroesseSetzen(400, 150)
        d.Drehen(180)

        d = Dreieck()
        d.FarbeSetzen("weiss")
        d.PositionSetzen(425, 200)
        d.GroesseSetzen(330, 100)
        d.Drehen(180)

        d = Dreieck()
        d.FarbeSetzen("grau")
        d.PositionSetzen(200, 125)
        d.GroesseSetzen(200, 50)
        d.Drehen(180)

        d = Dreieck()
        d.FarbeSetzen("gelb")
        d.PositionSetzen(350, 75)
        d.GroesseSetzen(300, 50)

        d = Dreieck()
        d.FarbeSetzen("grau")
        d.PositionSetzen(200, 125)
        d.GroesseSetzen(100, 425)

        d = Dreieck()
        d.FarbeSetzen("grau")
        d.PositionSetzen(500, 125)
        d.GroesseSetzen(100, 425)

        r = Rechteck()
        r.PositionSetzen(200, 125)
        r.FarbeSetzen("grau")
        r.GroesseSetzen(300, 425)

        r = Rechteck()
        r.PositionSetzen(225, 400)
        r.FarbeSetzen("blau")
        r.GroesseSetzen(225, 100)

        self.einheitText = "°C"
        self.einheit = Text()
        self.einheit.FarbeSetzen("weiss")
        self.einheit.PositionSetzen(250, 425)
        self.einheit.TextSetzen("Einheit: " + self.einheitText)

        self.aktTemp = 23
        self.aktuell = Text()
        self.aktuell.FarbeSetzen("weiss")
        self.aktuell.PositionSetzen(250, 450)
        self.aktuell.TextSetzen("aktuell: " + str(int(self.aktTemp)))

        self.zielTemp = 100
        self.ziel = Text()
        self.ziel.FarbeSetzen("weiss")
        self.ziel.PositionSetzen(250, 475)
        self.ziel.TextSetzen("Ziel: " + str(int(self.zielTemp)))

        self.TaktdauerSetzen(500)
        self.Starten()
        

    def AktionAusfuehren(self):
        """
        Führt den Taktimpuls aus
        """
        if self.istEingeschaltet:
            if self.aktTemp < self.zielTemp:
                self.aktTemp += 1.0
                self.aktuell.TextSetzen("aktuell: " + str(int(self.aktTemp)))
            else:
                self.istEingeschaltet = False
        else:
            if (self.aktTemp > 23 and self.einheitText == "°C") or (self.aktTemp > 73.4 and self.einheitText == "F"):
                self.aktTemp -= 0.1
                self.aktuell.TextSetzen("aktuell: " + str(int(self.aktTemp)))

    def Anschalten(self):
        """
        Schaltet den Wasserkocher ein
        """
        self.istEingeschaltet = True

    def AnzeigeSetzen(self, einh, akt, zielT):
        """
        Setzt die Anzeigeparameter
        :param einh: Einheit
        :param akt: aktuelle Temperatur
        :param zielT: Zieltemperatur
        """
        self.aktTemp = akt
        self.aktuell.TextSetzen("aktuell: " + str(int(self.aktTemp)))
        self.einheitText = einh
        self.einheit.TextSetzen("Einheit: " + self.einheitText)
        self.zielTemp = zielT
        self.ziel.TextSetzen("Ziel: " + str(int(self.zielTemp)))



class Wasserkochersteuerung:
    """
    Die Klasse Wasserkochersteuerung.
    """
    def __init__(self):
        """
        Belegt die Attribute mit den Startwerten
        """
        self.aktuelleTemperatur = 23
        self.zielTemperatur = 100
        self.einheit = "°C"
        self.wka = Wasserkocheranzeige()
        self.Anzeigen()

    def TemperaturGeben(self):
        """
        Gibt die aktuelle Temperatur zurück
        :return: aktuelle Temperatur
        """
        return self.aktuelleTemperatur

    def TemperaturAusgeben(self):
        """
        Gibt die aktuelle Temperatur auf der Konsole aus
        """
        print(f"{self.aktuelleTemperatur} {self.einheit}")

    def ZieltemperaturSetzen(self, zielTemperaturNeu):
        """
        Setzt die Zieltemperatur
        :param zielTemperaturNeu: neue Zieltemperatur
        """
        self.zielTemperatur = zielTemperaturNeu
        self.Anzeigen()

    def AktuelleTemperaturSetzen(self, aktTemperaturNeu):
        """
        Setzt die aktuelle Temperatur
        :param aktTemperaturNeu: neue aktuelle Temperatur
        """
        self.aktuelleTemperatur = aktTemperaturNeu
        self.Anzeigen()

    def TemperatureinheitWechseln(self, einheitNeu):
        """
        Wechselt die Temperatureinheit
        Mögliche Eingaben: "°C", "°F"
        :param einheitNeu: neue Temperatureinheit
        """
        if self.einheit != einheitNeu:
            faktor = 1.0
            summand = 0.0
            if self.einheit == "°F":
                faktor = 0.556
                summand = -17.778
                self.einheit = "°C"
            else:
                faktor = 1.8
                summand = 32
                self.einheit = "°F"
            self.aktuelleTemperatur = self.aktuelleTemperatur * faktor + summand
            self.zielTemperatur = self.zielTemperatur * faktor + summand
            self.Anzeigen()

    def Anzeigen(self):
        """
        Aktualisiert die Anzeige
        """
        self.wka.AnzeigeSetzen(self.einheit, self.aktuelleTemperatur, self.zielTemperatur)

    def Anschalten(self):
        """
        Schaltet den Wasserkocher ein
        """
        self.wka.Anschalten()

w = Wasserkochersteuerung()
w.Anschalten()