# -- coding: utf-8 --
"""
Created on Mon May  25 13:48:54 2020
@author: Albert Wiedemann
"""

from graphics_and_games_klassen import Figur, Ereignisbehandlung
import math
import random

class Jaeger(Figur):
    """
    Der Verfolger wird durch die Maus gesteuert.
    """
    
    def __init__(self, breite, hoehe):
        """
        Legt die Anzeigesymbole an und positioniert den Verfolger in der Mitte des Feldes.
        --Parameter breite Breite des Spielfelds
        --Parameter hoehe  Höhe des Spielfelds
        """
        super().__init__()
        self.breite = breite
        self.hoehe = hoehe
        self.PositionSetzen(breite / 2, hoehe / 2)
        self.WinkelSetzen(0)
        # Eigenes Aussehen des Jägers einbauen
        
    def AktionAusfuehren(self):
        """
        Bewegt den Verfolger um 10 Einheiten in der eingestellten Richtung.
        Wird der Rand erreicht, wird die Bewegung gestoppt.
        """
        xalt = self.XPositionGeben()
        yalt = self.YPositionGeben()
        self.Gehen(10)
        if ((self.XPositionGeben() < 0) and (self.XPositionGeben() < xalt) or (self.XPositionGeben() > self.breite) and (self.XPositionGeben() > xalt) or (self.YPositionGeben() < 0) and (self.YPositionGeben() < yalt) or (self.YPositionGeben() > self.hoehe) and (self.YPositionGeben() > yalt)):
            self.PositionSetzen(xalt, yalt)
            
    def MausGeklickt(self, x, y, button):
        """
        Setzt die Richtung des Verfolgers in Richtung des Mausklicks
        --Parameter button Maustaste (1-links, 2-Mausrad, 3-rechts, 4-Mausrad nach oben, 5-Mausrad nach unten)
        --Parameter pos Position des Mausklicks
        """
        self.WinkelSetzen(self.RichtungGeben(x, y))
        
    def RichtungGeben(self, xMaus, yMaus):
        """
        Wird bei Mausklick aufgerufen, um die Bewegungsrichtung des Verfolgers zu ermitteln.
        --Parameter xMaus x-Koordinate der Maus
        --Parameter yMaus y-Koordinate der Maus
        """
        dx = xMaus - self.XPositionGeben()
        dy = yMaus - self.YPositionGeben()
        richtung = 0
        if dx == 0:
            if dy > 0:
                richtung = 270
            elif dy < 0:
                richtung = 90
        elif dx > 0:
            richtung = - math.atan(dy/dx) * 180 / math.pi
        else:
            richtung = 180 - math.atan(dy/dx) * 180 / math.pi
        return richtung
    
class Beute (Figur):
    """
    Die Beute wandert zufällig über das Spielfeld.
    Sie wechselt ihre Richtung immer wahrscheinlicher, je länger sie gerade aus läuft.
    Als Darstellungssymbol hat sie einen Kreis
    """
    
    def __init__(self, breite, hoehe):
        """
        Legt das Anzeigesymbol an und wählt eine zufällige Startposition.
        --Parameter breite Breite des Spielfelds
        --Parameter hoehe  Höhe des Spielfelds
        """
        super().__init__()
        self.breite = breite
        self.hoehe = hoehe
        self.zaehler = 0
        self.PositionSetzen(random.random() * breite, random.random() * hoehe)
        self.WinkelSetzen(random.random() * 360)
        # Eigenes Aussehen der Beute einbauen
        
    def AktionAusfuehren(self):
        """
        Bewegt die Beute um 10 Einheiten in der eingestellten Richtung.
        Wird der Rand erreicht, wird die Richtung nach dem Reflexionsgesetzt
        gedreht. Außerdem wird sie spätestens nach 100 Schritten
        zufällig neu bestimmt; je länger die Geradesausbewegung,
        desto wahrscheinlicher ist die zufällige Änderung.
        """
        if random.random()*100 < self.zaehler:
            self.WinkelSetzen(random.random()*360)
            self.zaehler = 0
        self.Gehen(10)
        self.zaehler = self.zaehler + 1
        if (self.XPositionGeben() > self.breite) and (0 <= self.WinkelGeben()) and (self.WinkelGeben() < 90):
            self.WinkelSetzen(180 - self.WinkelGeben())
        elif (self.XPositionGeben() > self.breite) and (270 < self.WinkelGeben()) and (self.WinkelGeben() < 360):
            self.WinkelSetzen(540 - self.WinkelGeben())
        elif (self.XPositionGeben() < 0) and (90 < self.WinkelGeben()) and (self.WinkelGeben() <= 180):
            self.WinkelSetzen(180 - self.WinkelGeben())
        elif (self.XPositionGeben() < 0) and (180 < self.WinkelGeben()) and (self.WinkelGeben() < 270):
            self.WinkelSetzen(540 - self.WinkelGeben())
        if (self.YPositionGeben() < 0) and ((0 < self.WinkelGeben()) or (self.WinkelGeben() < 180)):
            self.WinkelSetzen(360 - self.WinkelGeben())
        elif (self.YPositionGeben() > self.hoehe) and (180 < self.WinkelGeben()) and (self.WinkelGeben() < 360):
            self.WinkelSetzen(360 - self.WinkelGeben())
        if self.Beruehrt():
            self.Umfaerben()
            
    def Umfaerben(self):
        """
        Färbt die erreichte Beute um.
        """
        # Eigene Form der Beute nach der Berührung einbauen.
        pass
    
class Verfolgung (Ereignisbehandlung):
    """
    Steuert das Verfolgungsspiel.
    """
    
    def __init__(self):
        """
        Legt die Objekte an.
        """
        super().__init__()
        Beute(600, 400)
        Jaeger(600, 400)
        self.TaktdauerSetzen(100)
        
    def MausGeklickt(self, x, y, button):
        pass
    
Verfolgung()
